@@ -18,6 +18,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
+
+int handle_codepage(int);
+void handle_fontfile(void);
 
 #define PACKED __attribute__ ((packed))
 /* Use this (instead of the above) to compile under MSDOS */
@@ -117,7 +121,7 @@ int main (int argc, char *argv[])
 	}
 
 	if (optc)
-	  handle_codepage();
+	  handle_codepage(0);
 	else
 	  handle_fontfile();
 
@@ -130,6 +134,7 @@ int main (int argc, char *argv[])
 	return (0);
 }
 
+void
 handle_fontfile(){
 	int i, j;
 
@@ -143,7 +148,7 @@ handle_fontfile(){
 	    exit(1);
 	}
 	if (optL)
-	  printf("FontFileHeader: id=%8.8s res=%8.8s num=%d typ=%c offset=%d\n\n",
+	  printf("FontFileHeader: id=%8.8s res=%8.8s num=%d typ=%c offset=%ld\n\n",
 		 FontFileHeader.id, FontFileHeader.res,
 		 FontFileHeader.num_pointers,
 		 FontFileHeader.p_type,
@@ -163,6 +168,7 @@ handle_fontfile(){
 	    break;
 }
 
+int
 handle_codepage(int more_to_come) {
         int j;
 	char outfile[20];
@@ -177,7 +183,7 @@ handle_codepage(int more_to_come) {
 	if (optL) {
 	    int t = CPEntryHeader.device_type;
 	    printf("CPEntryHeader: size=%d dev=%d [%s] name=%8.8s \
-codepage=%d\n\t\tres=%6.6s nxt=%d off_font=%d\n\n",
+codepage=%d\n\t\tres=%6.6s nxt=%ld off_font=%ld\n\n",
 		   CPEntryHeader.size,
 		   t, (t==1) ? "screen" : (t==2) ? "printer" : "?",
 		   CPEntryHeader.device_name,
@@ -204,13 +210,13 @@ codepage=%d\n\t\tres=%6.6s nxt=%d off_font=%d\n\n",
 	inpos = ftell(in);
 	if (inpos != CPEntryHeader.off_font && !optc) {
 	    if (optL)
-	      printf("pos=%d font at %d\n", inpos, CPEntryHeader.off_font);
+	      printf("pos=%ld font at %ld\n", inpos, CPEntryHeader.off_font);
 	    fseek(in, CPEntryHeader.off_font, SEEK_SET);
 	}
 
 	j = fread(&CPInfoHeader, 1, sizeof(CPInfoHeader), in);
 	if (j != sizeof(CPInfoHeader)) {
-	    printf("error reading CPInfoHeader - got %d chars\n");
+	    printf("error reading CPInfoHeader - got %d chars\n", j);
 	    exit(1);
 	}
 	if (optl) {
@@ -235,7 +241,7 @@ codepage=%d\n\t\tres=%6.6s nxt=%d off_font=%d\n\n",
 	fwrite(&CPInfoHeader, sizeof(CPInfoHeader), 1, out);
 	j = fwrite(fonts, 1, CPInfoHeader.size, out);
 	if (j != CPInfoHeader.size) {
-	    printf("error writing %s - wrote %d chars\n", j);
+	    printf("error writing %s - wrote %d chars\n", outfile, j);
 	    exit(1);
 	}
 	fclose(out);
@@ -252,7 +258,7 @@ codepage=%d\n\t\tres=%6.6s nxt=%d off_font=%d\n\n",
 	nexthdr = CPEntryHeader.off_nexthdr;
 	if (nexthdr == 0 || nexthdr == -1) {
 	    if (more_to_come) {
-		printf("mode codepages expected, but nexthdr=%d\n",
+		printf("mode codepages expected, but nexthdr=%ld\n",
 		       nexthdr);
 		exit(1);
 	    } else
@@ -262,7 +268,7 @@ codepage=%d\n\t\tres=%6.6s nxt=%d off_font=%d\n\n",
 	inpos = ftell(in);
 	if (inpos != CPEntryHeader.off_nexthdr) {
 	    if (optL)
-	      printf("pos=%d nexthdr at %d\n", inpos, nexthdr);
+	      printf("pos=%ld nexthdr at %ld\n", inpos, nexthdr);
 	    if (opta && !more_to_come) {
 		printf("no more code pages, but nexthdr != 0\n");
 		return 1;
