@@ -9,12 +9,13 @@
 #include <fcntl.h>
 #include <sys/ioctl.h>
 #include <linux/kd.h>
+#include "nls.h"
 
 extern int getfd(void);
 
 void
 usage(void) {
-    fprintf(stderr, "usage: getkeycodes\n");
+    fprintf(stderr, _("usage: getkeycodes\n"));
     exit(1);
 }
 
@@ -23,15 +24,19 @@ main(int argc, char **argv) {
     int fd, sc;
     struct kbkeycode a;
 
+    setlocale(LC_ALL, "");
+    bindtextdomain(PACKAGE, LOCALEDIR);
+    textdomain(PACKAGE);
+
     if (argc != 1)
       usage();
     fd = getfd();
-    printf("Plain scancodes xx (hex) versus keycodes (dec)\n");
-    printf("0 is an error; for 1-88 (0x01-0x58) scancode equals keycode\n");
+    printf(_("Plain scancodes xx (hex) versus keycodes (dec)\n"));
+    printf(_("0 is an error; for 1-88 (0x01-0x58) scancode equals keycode\n"));
 
     for(sc=88; sc<256; sc++) {
 	if (sc == 128)
-	  printf("\n\nEscaped scancodes e0 xx (hex)\n");
+	  printf(_("\n\nEscaped scancodes e0 xx (hex)\n"));
 	if (sc % 8 == 0) {
 	    if (sc < 128)
 	      printf("\n 0x%02x: ", sc);
@@ -51,7 +56,7 @@ main(int argc, char **argv) {
 	      printf("   -");
 	    else {
 		perror("KDGETKEYCODE");
-		fprintf(stderr, "failed to get keycode for scancode 0x%x\n",
+		fprintf(stderr, _("failed to get keycode for scancode 0x%x\n"),
 			sc);
 		exit(1);
 	    }
