@@ -9,12 +9,15 @@
 #include <sys/ioctl.h>
 #include <linux/types.h>
 #include <linux/kd.h>
+#include "nls.h"
+
+char *progname;
 
 extern int getfd(void);
 
 void
 usage(){
-    fprintf(stderr, "usage: kbd_mode [-a|-u|-k|-s]\n");
+    fprintf(stderr, _("usage: kbd_mode [-a|-u|-k|-s]\n"));
     exit(1);
 }
 
@@ -22,29 +25,36 @@ int
 main(int argc, char *argv[]){
         int fd, mode;
 
+	progname = argv[0];
+
+	setlocale(LC_ALL, "");
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	textdomain(PACKAGE);
+
 	fd = getfd();
 
 	if (argc == 1) {
 	    /* report mode */
 	    if (ioctl(fd, KDGKBMODE, &mode)) {
-	        perror("kbd_mode: error reading keyboard mode\n");
+		perror("KDGKBMODE");
+	        fprintf(stderr, _("kbd_mode: error reading keyboard mode\n"));
 		exit(1);
 	    }
 	    switch(mode) {
 	      case K_RAW:
-		printf("The keyboard is in raw (scancode) mode\n");
+		printf(_("The keyboard is in raw (scancode) mode\n"));
 		break;
 	      case K_MEDIUMRAW:
-		printf("The keyboard is in mediumraw (keycode) mode\n");
+		printf(_("The keyboard is in mediumraw (keycode) mode\n"));
 		break;
 	      case K_XLATE:
-		printf("The keyboard is in the default (ASCII) mode\n");
+		printf(_("The keyboard is in the default (ASCII) mode\n"));
 		break;
 	      case K_UNICODE:
-		printf("The keyboard is in Unicode (UTF-8) mode\n");
+		printf(_("The keyboard is in Unicode (UTF-8) mode\n"));
 		break;
 	      default:
-		printf("The keyboard is in some unknown mode\n");
+		printf(_("The keyboard is in some unknown mode\n"));
 	    }
 	    exit(1);
 	}
@@ -61,7 +71,8 @@ main(int argc, char *argv[]){
 	else
 	  usage();
 	if (ioctl(fd, KDSKBMODE, mode)) {
-		perror("kbd_mode: error setting keyboard mode\n");
+		perror("KDSKBMODE");
+		fprintf(stderr, _("%s: error setting keyboard mode\n"), progname);
 		exit(1);
 	}
 	exit(0);
