@@ -2,6 +2,7 @@
 #include <stdio.h>
 #include <string.h>
 #include "ksyms.h"
+#include "nls.h"
 
 static char *latin1_syms[] = {
 	"nul",
@@ -772,6 +773,7 @@ synonyms[] = {
 	{ "AltRLock",		"AltGr_Lock" },
 	{ "SCtrl",		"SControl" },
 	{ "Spawn_Console",	"KeyboardSignal" },
+	{ "Uncaps_Shift",	"CapsShift" },
 /* the names of the Greek letters are spelled differently
    in the iso-8859-7 and the Unicode standards */
 	{ "lambda",             "lamda" },
@@ -852,7 +854,7 @@ set_charset(char *charset) {
 	    }
 	}
 	if (!strcasecmp(charset, "unicode")) {
-	  fprintf (stderr, "unknown charset %s - ignoring charset request\n",
+	  fprintf (stderr, _("unknown charset %s - ignoring charset request\n"),
 		   charset);
 	  return 1;
 	}
@@ -886,25 +888,25 @@ ksymtocode(char *s)
 	for (i = 0; i < 256 - 160; i++)
 		if (!strcmp(s, latin2_syms[i])) {
 			if(!warn++)
-				fprintf(stderr, "assuming iso-8859-2 %s\n", s);
+				fprintf(stderr, _("assuming iso-8859-2 %s\n"), s);
 			return K(KT_LATIN, 160 + i);
 		}
 
 	for (i = 0; i < 256 - 160; i++)
 		if (!strcmp(s, latin3_syms[i])) {
 			if(!warn++)
-				fprintf(stderr, "assuming iso-8859-3 %s\n", s);
+				fprintf(stderr, _("assuming iso-8859-3 %s\n"), s);
 			return K(KT_LATIN, 160 + i);
 		}
 
 	for (i = 0; i < 256 - 160; i++)
 		if (!strcmp(s, latin4_syms[i])) {
 			if(!warn++)
-				fprintf(stderr, "assuming iso-8859-4 %s\n", s);
+				fprintf(stderr, _("assuming iso-8859-4 %s\n"), s);
 			return K(KT_LATIN, 160 + i);
 		}
 
-	fprintf(stderr, "unknown keysym '%s'\n", s);
+	fprintf(stderr, _("unknown keysym '%s'\n"), s);
 
 	return -1;
 }
