@@ -1,7 +1,7 @@
 /*
  * loadunimap.c - aeb
  *
- * Version 0.92
+ * Version 0.97
  */
 
 #include <errno.h>
@@ -14,24 +14,36 @@
 #include <linux/kd.h>
 #include <sys/ioctl.h>
 #include "paths.h"
+#include "psf.h"
+#include "nls.h"
 
-/* the two exported functions */
-void saveunicodemap(int fd, char *oufil);
+/* the three exported functions */
+void saveunicodemap(int fd, char *oufil);	/* save humanly readable */
 void loadunicodemap(int fd, char *ufil);
+void appendunicodemap(int fd, FILE *fp, int ct);/* append in psf style */
 
 extern char *malloc(), *realloc();
+extern char *progname;
 extern int getfd(void);
+extern int force;
 
 static char *unidirpath[] = { "", DATADIR "/" TRANSDIR "/", 0 };
 static char *unisuffixes[] = { "", ".uni", 0 };
 
 #ifdef MAIN
 int verbose = 0;
+int force = 0;
+char *progname;
 
 int
 main(int argc, char *argv[]) {
 	int fd;
-	char *progname = argv[0];
+
+	progname = argv[0];
+
+	setlocale(LC_ALL, "");
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	textdomain(PACKAGE);
 
 	fd = getfd();
 
@@ -44,7 +56,7 @@ main(int argc, char *argv[]) {
 	}
 		
 	if (argc > 2) {
-		fprintf(stderr, "usage: %s [-o map.orig] map-file\n",
+		fprintf(stderr, _("usage: %s [-o map.orig] map-file\n"),
 			progname);
 		exit(1);
 	}
@@ -94,7 +106,7 @@ void outlist() {
 	if (errno == ENOMEM && advice.advised_hashlevel < 100) {
 	    advice.advised_hashlevel++;
 #ifdef MAIN
-	    printf("trying hashlevel %d\n", advice.advised_hashlevel);
+	    printf(_("trying hashlevel %d\n"), advice.advised_hashlevel);
 #endif
 	    goto again;
 	}
@@ -109,7 +121,7 @@ void addpair(int fp, int un) {
 	listsz += 4096;
 	list = realloc((char *)list, listsz);
 	if (!list) {
-	    fprintf(stderr, "loadunimap: out of memory\n");
+	    fprintf(stderr, _("%s: out of memory\n"), progname);
 	    exit(1);
 	}
     }
@@ -134,13 +146,13 @@ loadunicodemap(int fd, char *tblname) {
     }
 
     if (verbose)
-      printf("Loading unicode map from file %s\n", pathname);
+      printf(_("Loading unicode map from file %s\n"), pathname);
 
     while ( fgets(buffer, sizeof(buffer), mapf) != NULL ) {
 	if ( (p = strchr(buffer, '\n')) != NULL )
 	  *p = '\0';
 	else
-	  fprintf(stderr, "loadunimap: %s: Warning: line too long\n", tblname);
+	  fprintf(stderr, _("%s: %s: Warning: line too long\n"), progname, tblname);
 
 	p = buffer;
 
@@ -162,7 +174,7 @@ loadunicodemap(int fd, char *tblname) {
 
 	fp0 = strtol(p, &p1, 0);
 	if (p1 == p) {
-	    fprintf(stderr, "Bad input line: %s\n", buffer);
+	    fprintf(stderr, _("Bad input line: %s\n"), buffer);
 	    exit(EX_DATAERR);
 	}
 	p = p1;
@@ -173,7 +185,7 @@ loadunicodemap(int fd, char *tblname) {
 	    p++;
 	    fp1 = strtol(p, &p1, 0);
 	    if (p1 == p) {
-		fprintf(stderr, "Bad input line: %s\n", buffer);
+		fprintf(stderr, _("Bad input line: %s\n"), buffer);
 		exit(EX_DATAERR);
 	    }
 	    p = p1;
@@ -183,20 +195,20 @@ loadunicodemap(int fd, char *tblname) {
 
 	if ( fp0 < 0 || fp0 >= fontlen ) {
 	    fprintf(stderr,
-		    "%s: Glyph number (0x%x) larger than font length\n",
+		    _("%s: Glyph number (0x%x) larger than font length\n"),
 		    tblname, fp0);
 	    exit(EX_DATAERR);
 	}
 	if ( fp1 && (fp1 < fp0 || fp1 >= fontlen) ) {
 	    fprintf(stderr,
-		    "%s: Bad end of range (0x%x)\n",
+		    _("%s: Bad end of range (0x%x)\n"),
 		    tblname, fp1);
 	    exit(EX_DATAERR);
 	}
 
 	if (fp1) {
-	    /* we have a range; expect the word "idem" or a Unicode range of the
-	       same length */
+	    /* we have a range; expect the word "idem" or a Unicode range
+	       of the same length */
 	    while (*p == ' ' || *p == '\t')
 	      p++;
 	    if (!strncmp(p, "idem", 4)) {
@@ -209,8 +221,8 @@ loadunicodemap(int fd, char *tblname) {
 		  p++;
 		if (*p != '-') {
 		    fprintf(stderr,
-			    "%s: Corresponding to a range of font positions, "
-			    "there should be a Unicode range\n",
+			    _("%s: Corresponding to a range of font positions, "
+			    "there should be a Unicode range\n"),
 			    tblname);
 		    exit(EX_DATAERR);
 		}
@@ -218,15 +230,15 @@ loadunicodemap(int fd, char *tblname) {
 		un1 = getunicode(&p);
 		if (un0 < 0 || un1 < 0) {
 		    fprintf(stderr,
-			    "%s: Bad Unicode range corresponding to "
-			    "font position range 0x%x-0x%x\n",
+			    _("%s: Bad Unicode range corresponding to "
+			    "font position range 0x%x-0x%x\n"),
 			    tblname, fp0, fp1);
 		    exit(EX_DATAERR);
 		}
 		if (un1 - un0 != fp1 - fp0) {
 		    fprintf(stderr,
-			    "%s: Unicode range U+%x-U+%x not of the same length"
-			    "as font position range 0x%x-0x%x\n",
+			    _("%s: Unicode range U+%x-U+%x not of the same"
+			      " length as font position range 0x%x-0x%x\n"),
 			    tblname, un0, un1, fp0, fp1);
 		    exit(EX_DATAERR);
 		}
@@ -243,21 +255,22 @@ loadunicodemap(int fd, char *tblname) {
 	while (*p == ' ' || *p == '\t')
 	  p++;
 	if (*p && *p != '#')
-	  fprintf(stderr, "%s: trailing junk (%s) ignored\n", tblname, p);
+	  fprintf(stderr, _("%s: trailing junk (%s) ignored\n"), tblname, p);
     }
 
     fpclose(mapf);
 
-    outlist();
+    if (listct == 0 && !force)
+      fprintf(stderr, _("%s: not loading empty unimap\n"
+	              "(if you insist: use option -f to override)\n"),
+	      progname);
+    else
+      outlist();
 }
 
-
-void
-saveunicodemap(int fd, char *oufil) {
-  FILE *fpo;
-  struct unipair *list = NULL;
+static struct unimapdesc
+getunicodemap(int fd) {
   struct unimapdesc descr;
-  int i;
 
   descr.entry_ct = 0;
   descr.entries = 0;
@@ -266,8 +279,12 @@ saveunicodemap(int fd, char *oufil) {
 	  perror("GIO_UNIMAP");
 	  exit(1);
       }
-      descr.entries = list = (struct unipair *)
+      descr.entries = (struct unipair *)
 	malloc(descr.entry_ct * sizeof(struct unipair));
+      if (descr.entries == NULL) {
+	  fprintf(stderr, _("%s: out of memory?\n"), progname);
+	  exit(1);
+      }
       if(ioctl(fd, GIO_UNIMAP, (unsigned long) &descr)) {
 	  perror("GIO_UNIMAP(2)");
 	  exit(1);
@@ -275,17 +292,63 @@ saveunicodemap(int fd, char *oufil) {
   }
 #ifdef MAIN
   fprintf(stderr, "# %d %s\n", descr.entry_ct,
-	 (descr.entry_ct == 1) ? "entry" : "entries");
+	 (descr.entry_ct == 1) ? _("entry") : _("entries"));
 #endif
 
+  return descr;
+}
+
+void
+saveunicodemap(int fd, char *oufil) {
+  FILE *fpo;
+  struct unimapdesc descr;
+  struct unipair *list;
+  int i;
+
   if ((fpo = fopen(oufil, "w")) == NULL) {
       perror(oufil);
       exit(1);
   }
+
+  descr = getunicodemap(fd);
+  list = descr.entries;
+
   for(i=0; i<descr.entry_ct; i++)
       fprintf(fpo, "0x%02x\tU+%04x\n", list[i].fontpos, list[i].unicode);
   fclose(fpo);
 
   if (verbose)
-    printf("Saved unicode map on `%s'\n", oufil);
+    printf(_("Saved unicode map on `%s'\n"), oufil);
+}
+
+static void
+appendunsignedshort(FILE *fp, unsigned short u) {
+  unsigned char unicodepair[2];
+
+  unicodepair[0] = (u & 0xff);
+  unicodepair[1] = ((u >> 8) & 0xff);
+  if (fwrite(unicodepair, sizeof(unicodepair), 1, fp) != 1) {
+    perror("appendunimap");
+    exit(1);
+  }
+}
+
+void
+appendunicodemap(int fd, FILE *fp, int fontsize) {
+  struct unimapdesc descr;
+  struct unipair *list;
+  int i, j;
+
+  descr = getunicodemap(fd);
+  list = descr.entries;
+
+  for(i=0; i<fontsize; i++) {
+    for(j=0; j<descr.entry_ct; j++)
+      if (list[j].fontpos == i)
+	appendunsignedshort(fp, list[j].unicode);
+    appendunsignedshort(fp, PSF_SEPARATOR);
+  }
+
+  if (verbose)
+     printf(_("Appended Unicode map\n"));
 }
