@@ -11,30 +11,31 @@
 #include <fcntl.h>
 #include <linux/kd.h>
 #include <sys/ioctl.h>
+#include "nls.h"
 
 void
 usage(void)
 {
-    fprintf(stderr, "\
-Usage:
-	setleds [-v] [-L] [-D] [-F] [[+|-][ num | caps | scroll ]]
-Thus,
-	setleds +caps -num
-will set CapsLock, clear NumLock and leave ScrollLock unchanged.
-The settings before and after the change (if any) are reported
-when the -v option is given or when no change is requested.
-Normally, setleds influences the vt flag settings
-(and these are usually reflected in the leds).
-With -L, setleds only sets the leds, and leaves the flags alone.
-With -D, setleds sets both the flags and the default flags, so
-that a subsequent reset will not change the flags.
-");
+    fprintf(stderr, _(
+"Usage:\n"
+"	setleds [-v] [-L] [-D] [-F] [[+|-][ num | caps | scroll ]]\n"
+"Thus,\n"
+"	setleds +caps -num\n"
+"will set CapsLock, clear NumLock and leave ScrollLock unchanged.\n"
+"The settings before and after the change (if any) are reported\n"
+"when the -v option is given or when no change is requested.\n"
+"Normally, setleds influences the vt flag settings\n"
+"(and these are usually reflected in the leds).\n"
+"With -L, setleds only sets the leds, and leaves the flags alone.\n"
+"With -D, setleds sets both the flags and the default flags, so\n"
+"that a subsequent reset will not change the flags.\n"
+));
     exit(1);
 }
 
     
 
-#define onoff(a) ((a) ? "on " : "off")
+#define onoff(a) ((a) ? _("on ") : _("off"))
 
 void
 report(leds) int leds;
@@ -63,17 +64,21 @@ main(argc,argv) int argc; char **argv;
     char *ap;
     struct led *lp;
 
+    setlocale(LC_ALL, "");
+    bindtextdomain(PACKAGE, LOCALEDIR);
+    textdomain(PACKAGE);
+
     if (ioctl(0, KDGETLED, &oleds)) {
 	perror("KDGETLED");
 	fprintf(stderr,
-	  "Error reading current led setting. Maybe stdin is not a VT?\n");
+	  _("Error reading current led setting. Maybe stdin is not a VT?\n"));
 	exit(1);
     }
 
     if (ioctl(0, KDGKBLED, &oflags)) {
 	perror("KDGKBLED");
 	fprintf(stderr,
-          "Error reading current flags setting. Maybe an old kernel?\n");
+          _("Error reading current flags setting. Maybe an old kernel?\n"));
 	exit(1);
     }
 
@@ -100,7 +105,7 @@ main(argc,argv) int argc; char **argv;
 	    nleds = 0xff;
 	    if (ioctl(0, KDSETLED, &nleds)) {
 		perror("KDSETLED");
-		fprintf(stderr, "Error resetting ledmode\n");
+		fprintf(stderr, _("Error resetting ledmode\n"));
 		exit(1);
 	    }
 	}
@@ -109,15 +114,15 @@ main(argc,argv) int argc; char **argv;
 	if (!optD && !optL && !optF)
 	  optD = optL = optF = 1;
 	if (optD) {
-	    printf("Current default flags:  ");
+	    printf(_("Current default flags:  "));
 	    report(odefflags);
 	}
 	if (optF) {
-	    printf("Current flags:          ");
+	    printf(_("Current flags:          "));
 	    report(oflags & 07);
 	}
 	if (optL) {
-	    printf("Current leds:           ");
+	    printf(_("Current leds:           "));
 	    report(oleds);
 	}
 	exit(0);
@@ -145,7 +150,7 @@ main(argc,argv) int argc; char **argv;
 		goto nxtarg;
 	    }
 	}
-	fprintf(stderr, "unrecognized argument: _%s_\n\n", ap);
+	fprintf(stderr, _("unrecognized argument: _%s_\n\n"), ap);
 	usage();
 
       nxtarg: ;
@@ -154,18 +159,18 @@ main(argc,argv) int argc; char **argv;
     if (optD) {
 	ndefflags = (odefflags & ~ndef) | nval;
 	if (verbose) {
-	    printf("Old default flags:    ");
+	    printf(_("Old default flags:    "));
 	    report(odefflags);
-	    printf("New default flags:    ");
+	    printf(_("New default flags:    "));
 	    report(ndefflags);
 	}
     }
     if (optF) {
 	nflags = ((oflags & ~ndef) | nval);
 	if (verbose) {
-	    printf("Old flags:            ");
+	    printf(_("Old flags:            "));
 	    report(oflags & 07);
-	    printf("New flags:            ");
+	    printf(_("New flags:            "));
 	    report(nflags & 07);
 	}
     }
@@ -178,9 +183,9 @@ main(argc,argv) int argc; char **argv;
     if (optL) {
 	nleds = (oleds & ~ndef) | nval;
 	if (verbose) {
-	    printf("Old leds:             ");
+	    printf(_("Old leds:             "));
 	    report(oleds);
-	    printf("New leds:             ");
+	    printf(_("New leds:             "));
 	    report(nleds);
 	}
 	if (ioctl(0, KDSETLED, nleds)) {
