@@ -3,6 +3,7 @@
 #include <linux/kd.h>
 #include <stdio.h>
 #include <sys/ioctl.h>
+#include "nls.h"
 
 #define CSI		155
 #define CSI_ERSATZ	126
@@ -14,7 +15,7 @@ leave(n) int n; {
   printf("\0338");		/* restore attributes */
   if (ioctl(0,PIO_SCRNMAP,obuf)) {
       perror("PIO_SCRNMAP");
-      fprintf(stderr, "failed to restore original translation table\n");
+      fprintf(stderr, _("failed to restore original translation table\n"));
       exit(1);
   }
   exit(n);
@@ -24,9 +25,14 @@ int
 main (argc,argv) int argc; char **argv; {
   unsigned char i, j, k;
 
+  setlocale(LC_ALL, "");
+  bindtextdomain(PACKAGE, LOCALEDIR);
+  textdomain(PACKAGE);
+
   if (argc != 1) {
-      fprintf(stderr, "usage: showfont
-(probably after loading a font with `setfont font')\n");
+      fprintf(stderr,
+	      _("usage: showfont\n"
+		"(probably after loading a font with `setfont font')\n"));
       exit(1);
   }
 
@@ -51,7 +57,7 @@ main (argc,argv) int argc; char **argv; {
   nbuf[10] = 0;
   if (ioctl(0,PIO_SCRNMAP,nbuf)) {
       perror("PIO_SCRNMAP");
-      fprintf(stderr, "cannot change translation table\n");
+      fprintf(stderr, _("cannot change translation table\n"));
       exit(1);
   }
 
@@ -77,7 +83,7 @@ main (argc,argv) int argc; char **argv; {
   nbuf[CSI_ERSATZ] = nbuf[CSI];
   if (ioctl(0,PIO_SCRNMAP,nbuf)) {
       perror("PIO_SCRNMAP");
-      fprintf(stderr, "cannot change translation table??\n");
+      fprintf(stderr, _("cannot change translation table??\n"));
       leave(1);
   }
 
