@@ -7,13 +7,12 @@
 #include <linux/kd.h>
 #include <linux/keyboard.h>
 #include <sys/ioctl.h>
+#include "nls.h"
 
-#define VERSION "0.96"
+#define VERSION "0.98"
 
 int tmp;	/* for debugging */
 
-#define OPTION "medium raw"
-
 extern int getfd();
 int fd;
 int oldkbmode;
@@ -40,12 +39,12 @@ void get_mode(void) {
 	  case K_UNICODE:
 	    m = "UNICODE"; break;
 	  default:
-	    m = "?UNKNOWN?"; break;
+	    m = _("?UNKNOWN?"); break;
 	}
-	printf("kb mode was %s\n", m);
+	printf(_("kb mode was %s\n"), m);
 	if (oldkbmode != K_XLATE) {
-	    printf("[ if you are trying this under X, it might not work\n");
-	    printf("since the X server is also reading /dev/console ]\n");
+	    printf(_("[ if you are trying this under X, it might not work\n"
+		     "since the X server is also reading /dev/console ]\n"));
 	}
 	printf("\n");
 }
@@ -55,14 +54,13 @@ void clean_up(void) {
 		perror("KDSKBMODE");
 		exit(1);
 	}
-	tmp = tcsetattr(fd, 0, &old);
-	if (tmp)
-		printf("tcsetattr = %d\n", tmp);
+	if (tcsetattr(fd, 0, &old) == -1)
+		perror("tcsetattr");
 	close(fd);
 }
 
 void die(int x) {
-	printf("caught signal %d, cleaning up...\n", x);
+	printf(_("caught signal %d, cleaning up...\n"), x);
 	clean_up();
 	exit(1);
 }
@@ -73,18 +71,17 @@ void watch_dog(int x) {
 }
 
 void usage(void) {
-	fprintf(stderr, "\
-showkey version " VERSION " (" OPTION ")
-
-usage: showkey [options...]
-
-valid options are:
-
-	-h --help	display this help text
-	-a --ascii	display the decimal/octal/hex values of the keys
-	-s --scancodes	display only the raw scan-codes
-	-k --keycodes	display only the interpreted keycodes (default)
-");
+	fprintf(stderr, _(
+"showkey version %s\n\n"
+"usage: showkey [options...]\n"
+"\n"
+"valid options are:\n"
+"\n"
+"	-h --help	display this help text\n"
+"	-a --ascii	display the decimal/octal/hex values of the keys\n"
+"	-s --scancodes	display only the raw scan-codes\n"
+"	-k --keycodes	display only the interpreted keycodes (default)\n"
+), VERSION);
 	exit(1);
 }
 
@@ -106,6 +103,10 @@ main (int argc, char *argv[]) {
 	unsigned char buf[16];
 	int i, n;
 
+	setlocale(LC_ALL, "");
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	textdomain(PACKAGE);
+
 	while ((c = getopt_long(argc, argv,
 		short_opts, long_opts, NULL)) != -1) {
 		switch (c) {
@@ -131,12 +132,10 @@ main (int argc, char *argv[]) {
 		/* no mode and signal and timer stuff - just read stdin */
 	        fd = 0;
 
-		tmp = tcgetattr(fd, &old);
-		if (tmp)
-			printf("tcgetattr = %d\n", tmp);
-		tmp = tcgetattr(fd, &new);
-		if (tmp)
-			printf("tcgetattr = %d\n", tmp);
+		if (tcgetattr(fd, &old) == -1)
+			perror("tcgetattr");
+		if (tcgetattr(fd, &new) == -1)
+			perror("tcgetattr");
 
 		new.c_lflag &= ~ (ICANON | ISIG);
 		new.c_lflag |= (ECHO | ECHOCTL);
@@ -144,11 +143,10 @@ main (int argc, char *argv[]) {
 		new.c_cc[VMIN] = 1;
 		new.c_cc[VTIME] = 0;
 
-		tmp = tcsetattr(fd, TCSAFLUSH, &new);
-		if (tmp)
-			printf("tcsetattr = %d\n", tmp);
-		printf("\nPress any keys - "
-		       "Ctrl-D will terminate this program\n\n");
+		if (tcsetattr(fd, TCSAFLUSH, &new) == -1)
+			perror("tcgetattr");
+		printf(_("\nPress any keys - "
+		         "Ctrl-D will terminate this program\n\n"));
 
 		while (1) {
 			n = read(fd, buf, 1);
@@ -159,9 +157,8 @@ main (int argc, char *argv[]) {
 				break;
 		}
 
-		tmp = tcsetattr(fd, 0, &old);
-		if (tmp)
-                	printf("tcsetattr = %d\n", tmp);
+		if (tcsetattr(fd, 0, &old) == -1)
+			perror("tcsetattr");
 		exit(0);
 	}
 
@@ -200,28 +197,24 @@ main (int argc, char *argv[]) {
 	signal(SIGTTOU, die);
 
 	get_mode();
-	tmp = tcgetattr(fd, &old);
-	if (tmp)
-		printf("tcgetattr = %d\n", tmp);
-	tmp = tcgetattr(fd, &new);
-	if (tmp)
-		printf("tcgetattr = %d\n", tmp);
+	if (tcgetattr(fd, &old) == -1)
+		perror("tcgetattr");
+	if (tcgetattr(fd, &new) == -1)
+		perror("tcgetattr");
 
 	new.c_lflag &= ~ (ICANON | ECHO | ISIG);
 	new.c_iflag = 0;
 	new.c_cc[VMIN] = sizeof(buf);
 	new.c_cc[VTIME] = 1;	/* 0.1 sec intercharacter timeout */
 
-	tmp = tcsetattr(fd, TCSAFLUSH, &new);
-	if (tmp)
-		printf("tcsetattr = %d\n", tmp);
-	if (ioctl(fd, KDSKBMODE,
-		show_keycodes ? K_MEDIUMRAW : K_RAW)) {
+	if (tcsetattr(fd, TCSAFLUSH, &new) == -1)
+		perror("tcsetattr");
+	if (ioctl(fd, KDSKBMODE, show_keycodes ? K_MEDIUMRAW : K_RAW)) {
 		perror("KDSKBMODE");
 		exit(1);
 	}
 
-	printf("press any key (program terminates after 10s of last keypress)...\n");
+	printf(_("press any key (program terminates 10s after last keypress)...\n"));
 	while (1) {
 		alarm(10);
 		n = read(fd, buf, sizeof(buf));
@@ -229,9 +222,10 @@ main (int argc, char *argv[]) {
 			if (!show_keycodes)
 				printf("0x%02x ", buf[i]);
 			else
-				printf("keycode %3d %s\n",
+				printf(_("keycode %3d %s\n"),
 					buf[i] & 0x7f,
-					buf[i] & 0x80 ? "release" : "press");
+					buf[i] & 0x80 ? _("release")
+				                      : _("press"));
 		}
 		if (!show_keycodes)
 			printf("\n");
